;----------------------------------------------------------------
; InThePicture Password Finder v0.1 (23 September 2002)
; Freeware, Open Source, GPL, Copyleft, whatever you want.
;----------------------------------------------------------------
;
; It's just a very small utility, quick-and-dirty coded,
; to recover files hidden by InThePicture steganography software 
;
; How to use: run my program. Select the BMP file with hidden 
; information data. That's all. Several things can happen:
;
;  - if the file is not a 24-bits BMP, you have an error message
;
;  - if the 4 first bytes of hidden data do not contain the
;    InThePicture specific signature, you have an error message
;
;  - if everything is okay, you have a MessageBox with a valid 
;    unlock key to extract your file.
;
; Limitations: you can retrieve the unlock key for just the first
;              hidden thing inside the BMP, can be a file or a
;              simple message. I realized I forgot to take in
;              account multiple file handling, and now I'm bored
;              and don't want to code anymore. Oh well.
;
; Guillermito
; guillermito@pipo.com
; http://www.guillermito.net
; September 23th 2002
;
; Assemble it with:
;      TASM32 /ml /m3 /z /t InThePicture_Password_Finder
;      TLINK32 -Tpe -aa InThePicture_Password_Finder,,,import32 


callW macro   x
extrn x:PROC
call x
endm

   .486
   .model flat

.data

openfilename_struct:
lStructSize       dd openfilename_struct_size
hwndOwner         dd 0
hInstance         dd 0
lpstrFilter       dd offset filter
lpstrCustomFilter dd 0
nMaxCustFilter    dd 0
nFilterIndex      dd 0
lpstrFile         dd offset namebuffer
nMaxFile          dd 255
lpstrFileTitle    dd 0
nMaxFileTitle     dd 32
lpstrInitialDir   dd 0
lpstrTitle        dd 0
Flags             dd 1000h+4h+200000h
nFileOffset       dw 0
nFileExtension    dw 0
lpstrDefExt       dd 0
lCustData         dd 0
lpfnHook          dd 0
lpTemplateName    dd 0
openfilename_struct_size equ $-offset openfilename_struct

filter            db "Bmp files *.bmp",0,"*.bmp",0,0
namebuffer	      db 255 dup(0)
result_title      db "InThePicture Password Finder by Guillermito", 0
no_bmp            db "This file does not have the 24-bits BMP signature",0
no_data           db "This BMP file does not seem to contain data hidden with InThePicture",0

file_size          dd ?
file_handle        dd ?
file_mem_buffer    dd ?
file_nb_bytes_read dd ?
pixels_data_start  dd ?

sub_key db 36h, 7bh, 4ch, 0f5h, 4ch, 0eah, 7, 2
        db 6, 33h, 2eh, 18h, 36h, 0eh, 8, 6
xor_key db 0c5h, 2, 0ceh, 61h, 0, 0edh

here_is_the_key    db "A valid key to extract the first hidden file is: "
scrambled_password db "0000AAAA000", 0

.code

programme:

;-------- choose a file -------------

push offset openfilename_struct
callW GetOpenFileNameA             ;choose file
test eax, eax
jz end

;-------- open it --------------

push 0
push 80h                          ;FILE_ATTRIBUTE_NORMAL
push 3                            ;OPEN_EXISTING
push 0
push 0
push 80000000h+40000000h          ;GENERIC_READ+GENERIC_WRITE
push dword ptr [lpstrFile]
callW CreateFileA                 ;open file
inc eax
jz end
dec eax
mov file_handle, eax

;---------- get its size ---------

push 0
push dword ptr file_handle
callW GetFileSize                 ;get file size in bytes
inc eax
jz close_file
dec eax
mov file_size, eax

;--------- alloc memory for file --------

mov eax, file_size
add eax, 1024
push eax                         ;number of bytes to allocate
push 40h                         ;40h=LMEM_ZEROINIT
callW LocalAlloc
test eax, eax
jz close_file
mov file_mem_buffer, eax

;------- read entire file ------------

push 0
push offset file_nb_bytes_read
push file_size
push file_mem_buffer
push file_handle
callW ReadFile
test eax, eax
jz free_memory
mov eax, file_nb_bytes_read
cmp eax, file_size
jnz free_memory

;-------- find BMP signature ---------

mov esi, file_mem_buffer
lodsw
cmp ax, "MB"
jne trouble

;------- find number of bits per pixel ------

add esi, 8
lodsd
add eax, file_mem_buffer
mov pixels_data_start, eax
add esi, 14
lodsw
cmp ax, 24
je this_looks_like_a_bmp

;-------- not 24-bits BMP -----------

trouble:
push 0
push offset result_title
push offset no_bmp
push 0
callW MessageBoxA
jmp free_memory

;------ Extract all the 2 less significant bits of the BMP -----------

this_looks_like_a_bmp:
mov eax, pixels_data_start
sub eax, file_mem_buffer
mov ecx, file_size
sub ecx, eax
shr ecx, 2
mov esi, pixels_data_start
mov edi, file_mem_buffer

extract_all:
push ecx
xor ebx, ebx
mov ecx, 4

extract_a_byte:
lodsb				;get byte
and al, 3			;eliminate everything except LSB
push ecx
dec cl
shl ecx, 1
rol al, cl			;put the bit where it belongs in the byte
pop ecx
or bl, al			;along with its friends
loop extract_a_byte
mov al, bl

mov dl, bl
and bl, 11b
and dl, 1100b       ;put them back in the good order
rol bl, 2           ;they are 12 34 78 56
ror dl, 2
or bl, dl
and al, 11110000b
or al, bl
  
stosb
pop ecx
loop extract_all

;---- descramble sig "ITP!" ---------

mov esi, 0      ;offset
mov ecx, 8      ;size
call descramble

;-------- check signature ---------

mov eax, [esi-8]
cmp eax, "!PTI"
je file_is_ITP_container

;-------- not ITP container -----------

push 0
push offset result_title
push offset no_data
push 0
callW MessageBoxA
jmp free_memory

;---- descramble "File/Mess" + key + size of path

file_is_ITP_container:
mov esi, 8      ;offset
mov ecx, 16     ;size
call descramble

;------ get scrambled key -----

sub esi, 8

;----- start with third field (3 digits) ---------

xor eax, eax
mov al, [esi]
sub al, 22h
xor al, 22h
xor ebx, ebx
mov bl, [esi+2]                        ;get the values
sub bl, 0c2h                           ;and descramble them
xor bl, 0c2h
and bl, 00001111b
rol ebx, 8
or eax, ebx

mov ebx, 10
xor edx, edx
div ebx
mov ebp, offset scrambled_password      ;transform them in base 10
add byte ptr [ebp+10], dl
xor edx, edx
div ebx
add byte ptr [ebp+9], dl
add byte ptr [ebp+8], al

;----- then get first field (4 digits) ---------

xor eax, eax
mov al, [esi+4]
sub al, 0c6h
xor al, 0c6h
shl eax, 2
xor ebx, ebx
mov bl, [esi+2]                        ;get the values
sub bl, 0c2h                           ;and descramble them
xor bl, 0c2h
and bl, 11110000b
ror bl, 2
add eax, ebx

mov ebx, 10
xor edx, edx
div ebx
add byte ptr [ebp+3], dl                ;transform them in base 10
xor edx, edx
div ebx
add byte ptr [ebp+2], dl
xor edx, edx
div ebx
add byte ptr [ebp+1], dl
add byte ptr [ebp], al

;----- get middle field (4 letters) ---------

xor eax, eax
mov al, [esi+1]
sub al, 0c4h
xor al, 0c4h
push eax
and al, 00111111b
call transform_ASCII
add byte ptr [ebp+7], al

pop eax
and al, 11000000b
ror eax, 6
mov bl, [esi+3]
sub bl, 0a4h
xor bl, 0a4h
push ebx
and bl, 00001111b
rol bl, 2
or al, bl
call transform_ASCII
add byte ptr [ebp+6], al

pop ebx
and bl, 11110000b
ror bl, 4
mov al, [esi+5]
push eax
and al, 00000011b
rol al, 4
or al, bl
call transform_ASCII
add byte ptr [ebp+5], al

pop eax
and al, 11111100b
ror al, 2
call transform_ASCII
add byte ptr [ebp+4], al

;-------- show the valid key -----------

push 0
push offset result_title
push offset here_is_the_key
push 0
callW MessageBoxA

;------- close memory and file ----------------

free_memory:
push file_mem_buffer
callW LocalFree

;----------- exit ----------

close_file:
push dword ptr file_handle
callW CloseHandle

end:
push -1
callW ExitProcess

;------ descramble ---------

descramble:
add esi, file_mem_buffer
mov edi, esi
xor ebx, ebx
xor edx, edx

descramble_loop:
lodsb
mov ebp, offset sub_key
sub al, byte ptr [ebp+ebx]
inc ebx
cmp ebx, 16
jne not_end_sub_key
xor ebx, ebx
not_end_sub_key:
mov ebp, offset xor_key
xor al, byte ptr [ebp+edx]
inc edx
cmp edx, 6
jne not_end_xor_key
xor edx, edx
not_end_xor_key:
stosb
loop descramble_loop
ret

;----- transform in ASCII key ------

transform_ASCII:
cmp al, 19h
ja highercase
add al, "a"-"A"
jmp lowercase
highercase:
sub al, 20h
lowercase:
ret

end programme
